#!/usr/bin/env python

'''****************************************************************************

*   Program - Convert To GFA format
*   Author - Mayank Pahadia


 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.

****************************************************************************'''

'''****************************************************************************

*   To run the program, pass three arguments with the python script on command line.
*   For example - python convertToGFA.py inputFileName outputFileName kmerSize

*   Logic - It reads through the fasta file with all the unitigs information 
*   and link information and outputs it in the GFA format. 


****************************************************************************'''


import sys
import argparse

def write_segment(name,segment,optional,g,links):
    add = ""
    add += "S\t" #for segment
    add += name #id of segment
    add += "\t"
    add += segment #segment itself
    add += "\t"
    for i in optional: #optional tags
        add+=i
        add+="\t"
    #adding Segment to the file
    g.write(add.strip()+"\n")
    for j in links: #adding all the links of the current segment to the GFA file
        g.write(j)


def main():
    parser = argparse.ArgumentParser(description="Convert a bcalm-generated FASTA to a GFA.",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('inputFilename', help='Input FASTA file')
    parser.add_argument('outputFilename', help='Output GFA file')
    parser.add_argument('kmerSize', type=int, help='k-mer length')
    parser.add_argument('-s', '--single-directed', action='store_true',
                        help='Avoid outputting the whole skew-simmetric graph and output only one edge between two nodes',
                        dest='single_directed')

    args = parser.parse_args()

    with open(args.inputFilename) as f:
        #name stores the id of the unitig
        #optional is a list which stores all the optional tags of a segment
        #links stores all the link information about a segment
        name = ""
        optional=[]
        links=[]
        g = open(args.outputFilename,'w')
        #adding Header to the file
        k = int(args.kmerSize)
        g.write('H\tVN:Z:1.0\tks:i:%d\n' %k) # includes the k-mer size
        print("GFA file open")
        #firstLine is for implemetation purpose so that we don't add some garbage value to the output file.
        firstLine = 0
        #segment stores the segment till present, in a fasta file, segment can be on many lines, hence we need to get the whole segment from all the lines
        segment = ""
        for line in f:
            line = line.replace("\n","")
            if(line[0]!=">"):
                #segment might be in more than one line, hence we get the whole segment first, and then put it in the GFA file.
                segment += line
            if(line[0]==">"):
                if(firstLine!=0):#if it's not the firstline in the input file, we store the input in GFA format in the output file
                    write_segment(name,segment,optional,g,links)
                    segment = ""

                firstLine = 1
                #once the previous segment and it's information has been stored, we start the next segment and it's information
                a = line.split(" ")
                name=a[0][1:] #get the id
                optional=[]
                links = []
                #we skip the first value because the first value is ">ID"
                for i in range(1,len(a)):
                    #we need this because the line can end with a space, hence we get one extra value in our list.
                    if(a[i]==""):
                        continue
                    if(a[i][0:2] == "MA"): #previous bcalm2 versions had "MA=[xxx]" optional tag as well, kept it just for compatibility, and reformated
                        optional.append(a[i][0:2]+":f:"+a[i][2:])
                    elif(a[i][0:2] == "L:"): #for links
                        b = a[i].split(":")
                        k1 = int(args.kmerSize)-1
                        if args.single_directed:
                            if name < b[2]:
                                links.append("L\t"+name+"\t"+b[1]+"\t"+b[2]+"\t"+b[3]+"\t"+str(k1)+"M\n")
                            elif name == b[2] and not (b[1] == b[3] == '-'): # manage links between the same unitig
                                links.append("L\t"+name+"\t"+b[1]+"\t"+b[2]+"\t"+b[3]+"\t"+str(k1)+"M\n")
                        else:
                            links.append("L\t"+name+"\t"+b[1]+"\t"+b[2]+"\t"+b[3]+"\t"+str(k1)+"M\n")
                    else: #all the other optional tags
                        optional.append(a[i])


        #we will miss the last one, because it won't go into the if condition - if(line[0]==">") and hence won't add the segment to the file.
        write_segment(name,segment,optional,g,links)
        print("done")
        g.close()

if __name__ == "__main__":
    main()



# take an assembly and a reference genome.
# determines whether each unitig is either fully erroneous or fully genomic, but that'd be an assembly error if it was both
import sys
fasta = open(sys.argv[1])
genome = [
"CTGTCGCGGGGAATTGTGGGGCGGACCACGCTCTGGCTAACGAGCTACCGTTTCCTTTAACCTGCCAGACGGTGACCAGGGCCGTTCGGC",
"GCCGAACGGCCCTGGTCACCGTCTGGCAGGTTAAAGGAAACGGTAGCTCGTTAGCCAGAGCGTGGTCCGCCCCACAATTCCCCGCGACAG"
]
id=""
k=31
for line in fasta:
    if line.startswith('>'):
        #print abundance
        id=line
    else:
        seq = line.strip()
        is_error = False
        is_genome = False
        for i in xrange(len(seq)-k+1):
            kmer = seq[i:i+k]
            if kmer not in genome[0] and kmer not in genome[1]:
                is_error = True
            else:
                is_genome = True

        if is_error and is_genome:
            print "mix of erroneous kmers and true genomic kmers inside the same assembled contig!!"
            print id
            print seq
            exit(1)
        else:
            print id,is_error,is_genome



#!/usr/bin/env python
import sys
max_mem = 0
for line in sys.stdin: 
    s = line.replace(']','').split()
    if 'MB' not in s:
        continue
    i=s.index('MB')
    if s[i-1].isdigit(): 
        mem=int(s[i-1])
        max_mem=max(mem,max_mem)

print max_mem



import sys
fasta = open(sys.argv[1])
id=""
for line in fasta:
    if line.startswith('>'):
        ma = line.find("MA=")
        abundance = float(line[ma+3:])
        #print abundance
        id=line
    else:
        for i in xrange(int(abundance)):
            print id,
            print line,




#very basic, one seq per line only, no pysam for max portability
#doesn't care if sequences are in revcomp or forward order
import sys

fasta1 = sys.argv[1]
fasta2 = sys.argv[2]

revcomp = lambda x: ''.join([{'A':'T','C':'G','G':'C','T':'A'}[B] for B in x][::-1])
def normalize(seq):
    rev = revcomp(seq)
    return min(rev,seq)

def read_seqs(fasta):
    seqs = set()
    for  line in open(fasta):
        if line[0] == '>': continue
        seqs.add(normalize(line.strip()))
    return seqs

s1 = read_seqs(fasta1)
s2 = read_seqs(fasta2)

if s1 == s2:
    sys.exit(0)

if len(s1) == len(s2) and len(s1) == 1:
    # special case, e.g. for read50 and genome10k, let's see if one is included in the other
    seq1= s1.copy().pop()
    seq2= s2.copy().pop()
    if seq1 in seq2 or seq2 in seq1:
        print("one seq is perfectly included in the other")
    else:
        print("BAD: one sequence doesn't exactly match the other")

print("NOT EQUAL: %d sequence(s) in %s not in %s" % (len(s1.difference(s2)), fasta1, fasta2))
sys.exit(1)



import sys
file=sys.argv[1]
for line in open(file):
     line=line.strip()
     if not line.startswith(">"):
        print(line.strip())
        continue
     if "L:" in line:
         s = line.split("L:")
     else:
         s = [line.strip()]
     print(s[0])




import sys

nb_ctg=0
nb_ilots=0
nb_connected=0

from collections import defaultdict
ctg_connect = defaultdict(bool)

assert(ctg_connect['dummy']==False)

def normalize(name):
    if name.endswith('\''):
        return name[:-1]
    return name

def header(name):
    assert(name[-1]==';')
    if ':' in name:
        res = name.split(':')[0]
    else:
        res = name.split(';')[0]
    return normalize(res)

with open(sys.argv[1]) as f:
    for line in f:
        if line.startswith('>'):
            line = line.strip()
            nb_ctg+=1
            connected = ':' in line 
            ctg_connect[header(line)] |= connected


for ctg in ctg_connect:
    if ctg_connect[ctg]:
        nb_connected += 1
    else:
        nb_ilots += 1

print "contigs input", nb_ctg/2, "connected", nb_connected, "ilots", nb_ilots
